@@ -14,13 +14,15 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+ * MA 02110-1301, USA.
  */
 
 #ifndef CHECK_LIST_H
 #define CHECK_LIST_H
 
+#include <stdbool.h>
+
 typedef struct List List;
 
 /* Create an empty list */
@@ -52,5 +54,8 @@ void check_list_free(List * lp);
 
 void check_list_apply(List * lp, void (*fp) (void *));
 
+/* Return true if the list contains the value, false otherwise */
+bool check_list_contains(List * lp, void *val);
+
 
 #endif /* CHECK_LIST_H */
