@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+ * MA 02110-1301, USA.
  */
 
 #include "../lib/libcompat.h"
@@ -60,6 +60,8 @@ static void srunner_run_init(SRunner * sr, enum print_output print_mode);
 static void srunner_run_end(SRunner * sr, enum print_output print_mode);
 static void srunner_iterate_suites(SRunner * sr,
                                    const char *sname, const char *tcname,
+				   const char *include_tags,
+				   const char *exclude_tags,
                                    enum print_output print_mode);
 static void srunner_iterate_tcase_tfuns(SRunner * sr, TCase * tc);
 static void srunner_add_failure(SRunner * sr, TestResult * tf);
@@ -96,6 +98,8 @@ static int waserror(int status, int expected_signal);
 
 static int alarm_received;
 static pid_t group_pid;
+static struct sigaction sigint_old_action;
+static struct sigaction sigterm_old_action;
 
 static void CK_ATTRIBUTE_UNUSED sig_handler(int sig_nr)
 {
@@ -105,6 +109,31 @@ static void CK_ATTRIBUTE_UNUSED sig_handler(int sig_nr)
             alarm_received = 1;
             killpg(group_pid, SIGKILL);
             break;
+        case SIGTERM:
+        case SIGINT:
+        {
+            pid_t own_group_pid;
+            int child_sig = SIGTERM;
+
+            if (sig_nr == SIGINT)
+            {
+                child_sig = SIGKILL;
+                sigaction(SIGINT, &sigint_old_action, NULL);
+            }
+            else
+            {
+                sigaction(SIGTERM, &sigterm_old_action, NULL);
+            }
+
+            killpg(group_pid, child_sig);
+
+            /* POSIX says that calling killpg(0)
+             * does not necessarily mean to call it on the callers
+             * group pid! */
+            own_group_pid = getpgrp();
+            killpg(own_group_pid, sig_nr);
+            break;
+        }
         default:
             eprintf("Unhandled signal: %d", __FILE__, __LINE__, sig_nr);
             break;
@@ -133,15 +162,22 @@ static void srunner_run_end(SRunner * sr,
 
 static void srunner_iterate_suites(SRunner * sr,
                                    const char *sname, const char *tcname,
+				   const char *include_tags,
+				   const char *exclude_tags,
                                    enum print_output CK_ATTRIBUTE_UNUSED
                                    print_mode)
 {
+    List *include_tag_lst;
+    List *exclude_tag_lst;
     List *slst;
     List *tcl;
     TCase *tc;
 
     slst = sr->slst;
 
+    include_tag_lst = tag_string_to_list(include_tags);
+    exclude_tag_lst = tag_string_to_list(exclude_tags);
+
     for(check_list_front(slst); !check_list_at_end(slst);
         check_list_advance(slst))
     {
@@ -164,12 +200,31 @@ static void srunner_iterate_suites(SRunner * sr,
             {
                 continue;
             }
+	    if (include_tags != NULL)
+	    {
+		if (!tcase_matching_tag(tc, include_tag_lst))
+		{
+		    continue;
+		}
+	    }
+	    if (exclude_tags != NULL)
+	    {
+		if (tcase_matching_tag(tc, exclude_tag_lst))
+		{
+		    continue;
+		}
+	    }
 
             srunner_run_tcase(sr, tc);
         }
 
         log_suite_end(sr, s);
     }
+
+    check_list_apply(include_tag_lst, free);
+    check_list_apply(exclude_tag_lst, free);
+    check_list_free(include_tag_lst);
+    check_list_free(exclude_tag_lst);
 }
 
 static void srunner_iterate_tcase_tfuns(SRunner * sr, TCase * tc)
@@ -450,7 +505,7 @@ static TestResult *tcase_run_tfun_fork(SRunner * sr, TCase * tc, TF * tfun,
     alarm_received = 0;
 
     if(timer_create(check_get_clockid(),
-                    NULL /* fire SIGALRM if timer expires */ ,
+                    NULL /* fire SIGALRM if timer expires */,
                     &timerid) == 0)
     {
         /* Set the timer to fire once */
@@ -685,7 +740,7 @@ enum fork_status srunner_fork_status(SRunner * sr)
 #if defined(HAVE_FORK) && HAVE_FORK==1
             return CK_FORK;
 #else /* HAVE_FORK */
-            eprintf("This version does not support fork", __FILE__, __LINE__);
+            /* Ignoring, as Check is not compiled with fork support. */
             return CK_NOFORK;
 #endif /* HAVE_FORK */
         }
@@ -700,7 +755,8 @@ void srunner_set_fork_status(SRunner * sr, enum fork_status fstat)
     /* If fork() is unavailable, do not allow a fork mode to be set */
     if(fstat != CK_NOFORK)
     {
-        eprintf("This version does not support fork", __FILE__, __LINE__);
+        /* Overriding, as Check is not compiled with fork support. */
+        fstat = CK_NOFORK;
     }
 #endif /* ! HAVE_FORK */
     sr->fstat = fstat;
@@ -713,12 +769,15 @@ void srunner_run_all(SRunner * sr, enum print_output print_mode)
                 print_mode);
 }
 
-void srunner_run(SRunner * sr, const char *sname, const char *tcname,
-                 enum print_output print_mode)
+void srunner_run_tagged(SRunner * sr, const char *sname, const char *tcname,
+			const char *include_tags, const char *exclude_tags,
+			enum print_output print_mode)
 {
 #if defined(HAVE_SIGACTION) && defined(HAVE_FORK)
-    struct sigaction old_action;
-    struct sigaction new_action;
+    static struct sigaction sigalarm_old_action;
+    static struct sigaction sigalarm_new_action;
+    static struct sigaction sigint_new_action;
+    static struct sigaction sigterm_new_action;
 #endif /* HAVE_SIGACTION && HAVE_FORK */
 
     /*  Get the selected test suite and test case from the
@@ -726,7 +785,11 @@ void srunner_run(SRunner * sr, const char *sname, const char *tcname,
     if(!tcname)
         tcname = getenv("CK_RUN_CASE");
     if(!sname)
-        sname = getenv("CK_RUN_SUITE");
+	sname = getenv("CK_RUN_SUITE");
+    if(!include_tags)
+	include_tags = getenv("CK_INCLUDE_TAGS");
+    if(!exclude_tags)
+	exclude_tags = getenv("CK_EXCLUDE_TAGS");
 
     if(sr == NULL)
         return;
@@ -736,18 +799,35 @@ void srunner_run(SRunner * sr, const char *sname, const char *tcname,
                 __FILE__, __LINE__, print_mode);
     }
 #if defined(HAVE_SIGACTION) && defined(HAVE_FORK)
-    memset(&new_action, 0, sizeof new_action);
-    new_action.sa_handler = sig_handler;
-    sigaction(SIGALRM, &new_action, &old_action);
+    memset(&sigalarm_new_action, 0, sizeof(sigalarm_new_action));
+    sigalarm_new_action.sa_handler = sig_handler;
+    sigaction(SIGALRM, &sigalarm_new_action, &sigalarm_old_action);
+
+    memset(&sigint_new_action, 0, sizeof(sigint_new_action));
+    sigint_new_action.sa_handler = sig_handler;
+    sigaction(SIGINT, &sigint_new_action, &sigint_old_action);
+
+    memset(&sigterm_new_action, 0, sizeof(sigterm_new_action));
+    sigterm_new_action.sa_handler = sig_handler;
+    sigaction(SIGTERM, &sigterm_new_action, &sigterm_old_action);
 #endif /* HAVE_SIGACTION && HAVE_FORK */
     srunner_run_init(sr, print_mode);
-    srunner_iterate_suites(sr, sname, tcname, print_mode);
+    srunner_iterate_suites(sr, sname, tcname, include_tags, exclude_tags,
+			   print_mode);
     srunner_run_end(sr, print_mode);
 #if defined(HAVE_SIGACTION) && defined(HAVE_FORK)
-    sigaction(SIGALRM, &old_action, NULL);
+    sigaction(SIGALRM, &sigalarm_old_action, NULL);
+    sigaction(SIGINT, &sigint_old_action, NULL);
+    sigaction(SIGTERM, &sigterm_old_action, NULL);
 #endif /* HAVE_SIGACTION && HAVE_FORK */
 }
 
+void srunner_run(SRunner * sr, const char *sname, const char *tcname,
+                 enum print_output print_mode)
+{
+    srunner_run_tagged(sr, sname, tcname, NULL, NULL, print_mode);
+}
+
 pid_t check_fork(void)
 {
 #if defined(HAVE_FORK) && HAVE_FORK==1
@@ -760,8 +840,8 @@ pid_t check_fork(void)
     }
     return pid;
 #else /* HAVE_FORK */
-    eprintf("This version does not support fork", __FILE__, __LINE__);
-    return 0;
+    /* Ignoring, as Check is not compiled with fork support. */
+    return -1;
 #endif /* HAVE_FORK */
 }
 
@@ -785,6 +865,7 @@ void check_waitpid_and_exit(pid_t pid CK_ATTRIBUTE_UNUSED)
     }
     exit(EXIT_SUCCESS);
 #else /* HAVE_FORK */
-    eprintf("This version does not support fork", __FILE__, __LINE__);
+    /* Ignoring, as Check is not compiled with fork support. */
+    exit(EXIT_FAILURE);
 #endif /* HAVE_FORK */
 }
